@@ -1,7 +1,10 @@
+//! libc syscalls supporting `rustix::fs`.
+
 use super::super::c;
 #[cfg(not(any(
     target_os = "dragonfly",
     target_os = "freebsd",
+    target_os = "illumos",
     target_os = "ios",
     target_os = "macos",
     target_os = "netbsd",
@@ -15,15 +18,11 @@ use super::super::conv::{
 };
 #[cfg(any(target_os = "android", target_os = "linux"))]
 use super::super::conv::{syscall_ret, syscall_ret_owned_fd, syscall_ret_ssize_t};
-use super::super::fd::BorrowedFd;
-#[cfg(not(target_os = "wasi"))]
-use super::super::fd::RawFd;
 #[cfg(any(target_os = "android", target_os = "fuchsia", target_os = "linux"))]
 use super::super::offset::libc_fallocate;
-#[cfg(not(any(target_os = "netbsd", target_os = "redox", target_os = "wasi")))]
-use super::super::offset::libc_fstatfs;
 #[cfg(not(any(
     target_os = "dragonfly",
+    target_os = "illumos",
     target_os = "ios",
     target_os = "macos",
     target_os = "netbsd",
@@ -35,6 +34,7 @@ use super::super::offset::libc_posix_fadvise;
     target_os = "android",
     target_os = "dragonfly",
     target_os = "fuchsia",
+    target_os = "illumos",
     target_os = "ios",
     target_os = "linux",
     target_os = "macos",
@@ -44,45 +44,64 @@ use super::super::offset::libc_posix_fadvise;
 )))]
 use super::super::offset::libc_posix_fallocate;
 use super::super::offset::{libc_fstat, libc_fstatat, libc_ftruncate, libc_lseek, libc_off_t};
-use super::super::time::Timespec;
+#[cfg(not(any(
+    target_os = "illumos",
+    target_os = "netbsd",
+    target_os = "redox",
+    target_os = "wasi"
+)))]
+use super::super::offset::{libc_fstatfs, libc_statfs};
+use crate::as_ptr;
+use crate::fd::BorrowedFd;
+#[cfg(not(target_os = "wasi"))]
+use crate::fd::RawFd;
+use crate::ffi::ZStr;
+#[cfg(any(target_os = "ios", target_os = "macos"))]
+use crate::ffi::ZString;
+#[cfg(not(target_os = "illumos"))]
+use crate::fs::Access;
 #[cfg(not(any(
     target_os = "dragonfly",
+    target_os = "illumos",
     target_os = "ios",
     target_os = "macos",
     target_os = "netbsd",
     target_os = "openbsd",
     target_os = "redox",
 )))]
-use super::Advice as FsAdvice;
+use crate::fs::Advice;
+#[cfg(not(any(
+    target_os = "dragonfly",
+    target_os = "illumos",
+    target_os = "netbsd",
+    target_os = "openbsd",
+    target_os = "redox"
+)))]
+use crate::fs::FallocateFlags;
+#[cfg(not(target_os = "wasi"))]
+use crate::fs::FlockOperation;
+#[cfg(any(target_os = "android", target_os = "linux"))]
+use crate::fs::MemfdFlags;
+#[cfg(not(any(
+    target_os = "illumos",
+    target_os = "netbsd",
+    target_os = "redox",
+    target_os = "wasi"
+)))]
+// not implemented in libc for netbsd yet
+use crate::fs::StatFs;
 #[cfg(not(any(
     target_os = "ios",
     target_os = "macos",
     target_os = "redox",
     target_os = "wasi",
 )))]
-use super::Dev;
-#[cfg(not(any(
-    target_os = "dragonfly",
-    target_os = "netbsd",
-    target_os = "openbsd",
-    target_os = "redox"
-)))]
-use super::FallocateFlags;
-#[cfg(not(target_os = "wasi"))]
-use super::FlockOperation;
+use crate::fs::{Dev, FileType};
+use crate::fs::{FdFlags, Mode, OFlags, Stat, Timestamps};
 #[cfg(any(target_os = "android", target_os = "linux"))]
-use super::MemfdFlags;
-#[cfg(not(any(target_os = "netbsd", target_os = "redox", target_os = "wasi")))]
-// not implemented in libc for netbsd yet
-use super::StatFs;
-use super::{Access, FdFlags, Mode, OFlags, Stat};
-#[cfg(any(target_os = "android", target_os = "linux"))]
-use super::{RenameFlags, ResolveFlags};
+use crate::fs::{RenameFlags, ResolveFlags};
 #[cfg(all(target_os = "linux", target_env = "gnu"))]
-use super::{Statx, StatxFlags};
-use crate::ffi::ZStr;
-#[cfg(any(target_os = "ios", target_os = "macos"))]
-use crate::ffi::ZString;
+use crate::fs::{Statx, StatxFlags};
 use crate::io::{self, OwnedFd, SeekFrom};
 #[cfg(not(target_os = "wasi"))]
 use crate::process::{Gid, Uid};
@@ -97,10 +116,10 @@ use core::ptr::null_mut;
 #[cfg(any(target_os = "ios", target_os = "macos"))]
 use {
     super::super::conv::nonnegative_ret,
-    super::{copyfile_state_t, CloneFlags, CopyfileFlags},
+    crate::fs::{copyfile_state_t, CloneFlags, CopyfileFlags},
 };
 #[cfg(not(target_os = "redox"))]
-use {super::super::offset::libc_openat, super::AtFlags};
+use {super::super::offset::libc_openat, crate::fs::AtFlags};
 
 #[cfg(not(target_os = "redox"))]
 pub(crate) fn openat(
@@ -111,7 +130,7 @@ pub(crate) fn openat(
 ) -> io::Result<OwnedFd> {
     unsafe {
         // Pass `mode` as a `c_uint` even if `mode_t` is narrower, since
-        // `libc_openat` is declared as a variadic function and narrrower
+        // `libc_openat` is declared as a variadic function and narrower
         // arguments are promoted.
         ret_owned_fd(libc_openat(
             borrowed_fd(dirfd),
@@ -122,6 +141,21 @@ pub(crate) fn openat(
     }
 }
 
+#[cfg(not(any(
+    target_os = "illumos",
+    target_os = "netbsd",
+    target_os = "redox",
+    target_os = "wasi"
+)))]
+#[inline]
+pub(crate) fn statfs(filename: &ZStr) -> io::Result<StatFs> {
+    unsafe {
+        let mut result = MaybeUninit::<StatFs>::uninit();
+        ret(libc_statfs(c_str(filename), result.as_mut_ptr()))?;
+        Ok(result.assume_init())
+    }
+}
+
 #[cfg(not(target_os = "redox"))]
 #[inline]
 pub(crate) fn readlinkat(dirfd: BorrowedFd<'_>, path: &ZStr, buf: &mut [u8]) -> io::Result<usize> {
@@ -248,7 +282,7 @@ pub(crate) fn statat(dirfd: BorrowedFd<'_>, path: &ZStr, flags: AtFlags) -> io::
     }
 }
 
-#[cfg(not(any(target_os = "redox", target_os = "emscripten")))]
+#[cfg(not(any(target_os = "emscripten", target_os = "illumos", target_os = "redox")))]
 pub(crate) fn accessat(
     dirfd: BorrowedFd<'_>,
     path: &ZStr,
@@ -279,14 +313,17 @@ pub(crate) fn accessat(
 pub(crate) fn utimensat(
     dirfd: BorrowedFd<'_>,
     path: &ZStr,
-    times: &[Timespec; 2],
+    times: &Timestamps,
     flags: AtFlags,
 ) -> io::Result<()> {
     unsafe {
+        // Assert that `Timestamps` has the expected layout.
+        let _ = core::mem::transmute::<Timestamps, [c::timespec; 2]>(times.clone());
+
         ret(c::utimensat(
             borrowed_fd(dirfd),
             c_str(path),
-            times.as_ptr(),
+            as_ptr(times).cast(),
             flags.bits(),
         ))
     }
@@ -307,7 +344,7 @@ pub(crate) fn chmodat(dirfd: BorrowedFd<'_>, path: &ZStr, mode: Mode) -> io::Res
     // Note that Linux's `fchmodat` does not have a flags argument.
     unsafe {
         // Pass `mode` as a `c_uint` even if `mode_t` is narrower, since
-        // `libc_openat` is declared as a variadic function and narrrower
+        // `libc_openat` is declared as a variadic function and narrower
         // arguments are promoted.
         syscall_ret(c::syscall(
             c::SYS_fchmodat,
@@ -362,12 +399,18 @@ pub(crate) fn chownat(
     target_os = "redox",
     target_os = "wasi",
 )))]
-pub(crate) fn mknodat(dirfd: BorrowedFd<'_>, path: &ZStr, mode: Mode, dev: Dev) -> io::Result<()> {
+pub(crate) fn mknodat(
+    dirfd: BorrowedFd<'_>,
+    path: &ZStr,
+    file_type: FileType,
+    mode: Mode,
+    dev: Dev,
+) -> io::Result<()> {
     unsafe {
         ret(c::mknodat(
             borrowed_fd(dirfd),
             c_str(path),
-            mode.bits(),
+            mode.bits() | file_type.as_raw_mode(),
             dev,
         ))
     }
@@ -421,18 +464,14 @@ pub(crate) fn copy_file_range(
 
 #[cfg(not(any(
     target_os = "dragonfly",
+    target_os = "illumos",
     target_os = "ios",
     target_os = "macos",
     target_os = "netbsd",
     target_os = "openbsd",
     target_os = "redox"
 )))]
-pub(crate) fn fadvise(
-    fd: BorrowedFd<'_>,
-    offset: u64,
-    len: u64,
-    advice: FsAdvice,
-) -> io::Result<()> {
+pub(crate) fn fadvise(fd: BorrowedFd<'_>, offset: u64, len: u64, advice: Advice) -> io::Result<()> {
     let offset = offset as i64;
     let len = len as i64;
 
@@ -481,6 +520,7 @@ pub(crate) fn fcntl_setfl(fd: BorrowedFd<'_>, flags: OFlags) -> io::Result<()> {
 #[cfg(not(any(
     target_os = "dragonfly",
     target_os = "freebsd",
+    target_os = "illumos",
     target_os = "ios",
     target_os = "macos",
     target_os = "netbsd",
@@ -566,7 +606,12 @@ pub(crate) fn fstat(fd: BorrowedFd<'_>) -> io::Result<Stat> {
     }
 }
 
-#[cfg(not(any(target_os = "netbsd", target_os = "redox", target_os = "wasi")))] // not implemented in libc for netbsd yet
+#[cfg(not(any(
+    target_os = "illumos",
+    target_os = "netbsd",
+    target_os = "redox",
+    target_os = "wasi"
+)))] // not implemented in libc for netbsd yet
 pub(crate) fn fstatfs(fd: BorrowedFd<'_>) -> io::Result<StatFs> {
     let mut statfs = MaybeUninit::<StatFs>::uninit();
     unsafe {
@@ -575,12 +620,18 @@ pub(crate) fn fstatfs(fd: BorrowedFd<'_>) -> io::Result<StatFs> {
     }
 }
 
-pub(crate) fn futimens(fd: BorrowedFd<'_>, times: &[Timespec; 2]) -> io::Result<()> {
-    unsafe { ret(c::futimens(borrowed_fd(fd), times.as_ptr())) }
+pub(crate) fn futimens(fd: BorrowedFd<'_>, times: &Timestamps) -> io::Result<()> {
+    unsafe {
+        // Assert that `Timestamps` has the expected layout.
+        let _ = core::mem::transmute::<Timestamps, [c::timespec; 2]>(times.clone());
+
+        ret(c::futimens(borrowed_fd(fd), as_ptr(times).cast()))
+    }
 }
 
 #[cfg(not(any(
     target_os = "dragonfly",
+    target_os = "illumos",
     target_os = "ios",
     target_os = "macos",
     target_os = "netbsd",
@@ -844,11 +895,11 @@ pub(crate) fn getpath(fd: BorrowedFd<'_>) -> io::Result<ZString> {
     // instead.
     let mut buf = vec![0; c::PATH_MAX as usize];
 
-    // From the macOS `fcntl` man page:
+    // From the [macOS `fcntl` man page]:
     // `F_GETPATH` - Get the path of the file descriptor `Fildes`. The argument
     //               must be a buffer of size `MAXPATHLEN` or greater.
     //
-    // https://developer.apple.com/library/archive/documentation/System/Conceptual/ManPages_iPhoneOS/man2/fcntl.2.html
+    // [macOS `fcntl` man page]: https://developer.apple.com/library/archive/documentation/System/Conceptual/ManPages_iPhoneOS/man2/fcntl.2.html
     unsafe {
         ret(c::fcntl(borrowed_fd(fd), c::F_GETPATH, buf.as_mut_ptr()))?;
     }
@@ -865,7 +916,7 @@ pub(crate) fn getpath(fd: BorrowedFd<'_>) -> io::Result<ZString> {
 
 #[cfg(any(target_os = "ios", target_os = "macos"))]
 pub(crate) fn fcntl_rdadvise(fd: BorrowedFd<'_>, offset: u64, len: u64) -> io::Result<()> {
-    // From the macOS `fcntl` man page:
+    // From the [macOS `fcntl` man page]:
     // `F_RDADVISE` - Issue an advisory read async with no copy to user.
     //
     // The `F_RDADVISE` command operates on the following structure which holds
@@ -878,7 +929,7 @@ pub(crate) fn fcntl_rdadvise(fd: BorrowedFd<'_>, offset: u64, len: u64) -> io::R
     // };
     // ```
     //
-    // https://developer.apple.com/library/archive/documentation/System/Conceptual/ManPages_iPhoneOS/man2/fcntl.2.html
+    // [macOS `fcntl` man page]: https://developer.apple.com/library/archive/documentation/System/Conceptual/ManPages_iPhoneOS/man2/fcntl.2.html
     let ra_offset = match offset.try_into() {
         Ok(len) => len,
         // If this conversion fails, the user is providing an offset outside
