@@ -1,10 +1,11 @@
+//! libc syscalls supporting `rustix::io`.
+
 use super::super::c;
 #[cfg(any(target_os = "android", target_os = "linux"))]
 use super::super::conv::syscall_ret_owned_fd;
 use super::super::conv::{
     borrowed_fd, no_fd, ret, ret_c_int, ret_discarded_fd, ret_owned_fd, ret_ssize_t,
 };
-use super::super::fd::{AsFd, BorrowedFd, RawFd};
 #[cfg(not(target_os = "wasi"))]
 use super::super::offset::libc_mmap;
 use super::super::offset::{libc_pread, libc_pwrite};
@@ -12,22 +13,20 @@ use super::super::offset::{libc_pread, libc_pwrite};
 use super::super::offset::{libc_preadv, libc_pwritev};
 #[cfg(all(target_os = "linux", target_env = "gnu"))]
 use super::super::offset::{libc_preadv2, libc_pwritev2};
-#[cfg(not(any(target_os = "redox", target_os = "wasi")))]
-use super::Advice as IoAdvice;
-#[cfg(target_os = "linux")]
-use super::MremapFlags;
-#[cfg(not(any(target_os = "ios", target_os = "macos", target_os = "wasi")))]
-use super::PipeFlags;
-use super::PollFd;
-#[cfg(not(target_os = "wasi"))]
-use super::{DupFlags, MapFlags, MprotectFlags, ProtFlags, Termios, Winsize};
-#[cfg(any(target_os = "android", target_os = "linux"))]
-use super::{EventfdFlags, UserfaultfdFlags};
-#[cfg(any(target_os = "android", target_os = "linux"))]
-use super::{MlockFlags, ReadWriteFlags};
+use crate::fd::{AsFd, BorrowedFd, RawFd};
 #[cfg(not(any(target_os = "fuchsia", target_os = "wasi")))]
 use crate::ffi::ZStr;
-use crate::io::{self, IoSlice, IoSliceMut, OwnedFd};
+#[cfg(not(any(target_os = "redox", target_os = "wasi")))]
+use crate::io::Advice;
+#[cfg(target_os = "linux")]
+use crate::io::MremapFlags;
+#[cfg(not(any(target_os = "ios", target_os = "macos", target_os = "wasi")))]
+use crate::io::PipeFlags;
+use crate::io::{self, IoSlice, IoSliceMut, OwnedFd, PollFd};
+#[cfg(not(target_os = "wasi"))]
+use crate::io::{DupFlags, MapFlags, MprotectFlags, MsyncFlags, ProtFlags, Termios, Winsize};
+#[cfg(any(target_os = "android", target_os = "linux"))]
+use crate::io::{EventfdFlags, MlockFlags, ReadWriteFlags, UserfaultfdFlags};
 use core::cmp::min;
 use core::convert::TryInto;
 use core::mem::MaybeUninit;
@@ -89,7 +88,7 @@ pub(crate) fn pwrite(fd: BorrowedFd<'_>, buf: &[u8], offset: u64) -> io::Result<
     Ok(nwritten as usize)
 }
 
-pub(crate) fn readv(fd: BorrowedFd<'_>, bufs: &[IoSliceMut]) -> io::Result<usize> {
+pub(crate) fn readv(fd: BorrowedFd<'_>, bufs: &mut [IoSliceMut]) -> io::Result<usize> {
     let nread = unsafe {
         ret_ssize_t(c::readv(
             borrowed_fd(fd),
@@ -112,7 +111,11 @@ pub(crate) fn writev(fd: BorrowedFd<'_>, bufs: &[IoSlice]) -> io::Result<usize>
 }
 
 #[cfg(not(target_os = "redox"))]
-pub(crate) fn preadv(fd: BorrowedFd<'_>, bufs: &[IoSliceMut], offset: u64) -> io::Result<usize> {
+pub(crate) fn preadv(
+    fd: BorrowedFd<'_>,
+    bufs: &mut [IoSliceMut],
+    offset: u64,
+) -> io::Result<usize> {
     // Silently cast; we'll get `EINVAL` if the value is negative.
     let offset = offset as i64;
     let nread = unsafe {
@@ -144,7 +147,7 @@ pub(crate) fn pwritev(fd: BorrowedFd<'_>, bufs: &[IoSlice], offset: u64) -> io::
 #[cfg(all(target_os = "linux", target_env = "gnu"))]
 pub(crate) fn preadv2(
     fd: BorrowedFd<'_>,
-    bufs: &[IoSliceMut],
+    bufs: &mut [IoSliceMut],
     offset: u64,
     flags: ReadWriteFlags,
 ) -> io::Result<usize> {
@@ -171,7 +174,7 @@ pub(crate) fn preadv2(
 #[inline]
 pub(crate) fn preadv2(
     fd: BorrowedFd<'_>,
-    bufs: &[IoSliceMut],
+    bufs: &mut [IoSliceMut],
     offset: u64,
     flags: ReadWriteFlags,
 ) -> io::Result<usize> {
@@ -270,12 +273,12 @@ pub(crate) unsafe fn close(raw_fd: RawFd) {
 }
 
 #[cfg(not(any(target_os = "redox", target_os = "wasi")))]
-pub(crate) fn madvise(addr: *mut c::c_void, len: usize, advice: IoAdvice) -> io::Result<()> {
+pub(crate) fn madvise(addr: *mut c::c_void, len: usize, advice: Advice) -> io::Result<()> {
     // On Linux platforms, `MADV_DONTNEED` has the same value as
     // `POSIX_MADV_DONTNEED` but different behavior. We remap it to a different
     // value, and check for it here.
     #[cfg(target_os = "linux")]
-    if let IoAdvice::LinuxDontNeed = advice {
+    if let Advice::LinuxDontNeed = advice {
         return unsafe { ret(c::madvise(addr, len, c::MADV_DONTNEED)) };
     }
 
@@ -293,7 +296,7 @@ pub(crate) fn madvise(addr: *mut c::c_void, len: usize, advice: IoAdvice) -> io:
 
     #[cfg(target_os = "android")]
     {
-        if let IoAdvice::DontNeed = advice {
+        if let Advice::DontNeed = advice {
             // Do nothing. Linux's `MADV_DONTNEED` isn't the same as
             // `POSIX_MADV_DONTNEED`, so just discard `MADV_DONTNEED`.
             Ok(())
@@ -303,11 +306,51 @@ pub(crate) fn madvise(addr: *mut c::c_void, len: usize, advice: IoAdvice) -> io:
     }
 }
 
+#[cfg(not(target_os = "wasi"))]
+pub(crate) unsafe fn msync(addr: *mut c::c_void, len: usize, flags: MsyncFlags) -> io::Result<()> {
+    let err = c::msync(addr, len, flags.bits());
+
+    // `msync` returns its error status rather than using `errno`.
+    if err == 0 {
+        Ok(())
+    } else {
+        Err(io::Error(err))
+    }
+}
+
 #[cfg(any(target_os = "android", target_os = "linux"))]
 pub(crate) fn eventfd(initval: u32, flags: EventfdFlags) -> io::Result<OwnedFd> {
     unsafe { syscall_ret_owned_fd(c::syscall(c::SYS_eventfd2, initval, flags.bits())) }
 }
 
+#[cfg(target_os = "linux")]
+#[inline]
+pub(crate) fn ioctl_blksszget(fd: BorrowedFd) -> io::Result<u32> {
+    let mut result = MaybeUninit::<c::c_uint>::uninit();
+    unsafe {
+        ret(libc::ioctl(
+            borrowed_fd(fd),
+            libc::BLKSSZGET as c::c_ulong,
+            result.as_mut_ptr(),
+        ))?;
+        Ok(result.assume_init() as u32)
+    }
+}
+
+#[cfg(target_os = "linux")]
+#[inline]
+pub(crate) fn ioctl_blkpbszget(fd: BorrowedFd) -> io::Result<u32> {
+    let mut result = MaybeUninit::<c::c_uint>::uninit();
+    unsafe {
+        ret(libc::ioctl(
+            borrowed_fd(fd),
+            libc::BLKPBSZGET as c::c_ulong,
+            result.as_mut_ptr(),
+        ))?;
+        Ok(result.assume_init() as u32)
+    }
+}
+
 #[cfg(not(target_os = "redox"))]
 pub(crate) fn ioctl_fionread(fd: BorrowedFd<'_>) -> io::Result<u64> {
     let mut nread = MaybeUninit::<c::c_int>::uninit();
@@ -335,7 +378,7 @@ pub(crate) fn isatty(fd: BorrowedFd<'_>) -> bool {
             c::ENOTTY => false,
 
             // Old Linux versions reportedly return `EINVAL`.
-            // https://man7.org/linux/man-pages/man3/isatty.3.html#ERRORS
+            // <https://man7.org/linux/man-pages/man3/isatty.3.html#ERRORS>
             #[cfg(any(target_os = "android", target_os = "linux"))]
             c::ENOTTY | c::EINVAL => false,
 
@@ -370,7 +413,7 @@ pub(crate) fn is_read_write(fd: BorrowedFd<'_>) -> io::Result<(bool, bool)> {
         } {
             0 => read = false,
             -1 => {
-                #[allow(unreachable_patterns)] // EAGAIN may equal EWOULDBLOCK
+                #[allow(unreachable_patterns)] // `EAGAIN` may equal `EWOULDBLOCK`
                 match errno().0 {
                     c::EAGAIN | c::EWOULDBLOCK => (),
                     c::ENOTSOCK => not_socket = true,
@@ -385,7 +428,7 @@ pub(crate) fn is_read_write(fd: BorrowedFd<'_>) -> io::Result<(bool, bool)> {
         // the write side is shut down.
         match unsafe { c::send(borrowed_fd(fd), [].as_ptr(), 0, c::MSG_DONTWAIT) } {
             -1 => {
-                #[allow(unreachable_patterns)] // EAGAIN may equal EWOULDBLOCK
+                #[allow(unreachable_patterns)] // `EAGAIN` may equal `EWOULDBLOCK`
                 match errno().0 {
                     c::EAGAIN | c::EWOULDBLOCK => (),
                     c::ENOTSOCK => (),
@@ -440,7 +483,7 @@ pub(crate) fn dup2_with(fd: BorrowedFd<'_>, new: &OwnedFd, flags: DupFlags) -> i
     target_os = "redox"
 ))]
 pub(crate) fn dup2_with(fd: BorrowedFd<'_>, new: &OwnedFd, _flags: DupFlags) -> io::Result<()> {
-    // Android 5.0 has dup3, but libc doesn't have bindings
+    // Android 5.0 has `dup3`, but libc doesn't have bindings
     dup2(fd, new)
 }
 
