@@ -95,13 +95,14 @@ pub(super) use c::openat64 as libc_openat;
 
 #[cfg(target_os = "fuchsia")]
 pub(super) use c::fallocate as libc_fallocate;
-#[cfg(any(target_os = "android", target_os = "linux",))]
+#[cfg(any(target_os = "android", target_os = "linux"))]
 pub(super) use c::fallocate64 as libc_fallocate;
 #[cfg(not(any(
     windows,
     target_os = "android",
     target_os = "dragonfly",
     target_os = "emscripten",
+    target_os = "illumos",
     target_os = "ios",
     target_os = "linux",
     target_os = "l4re",
@@ -167,22 +168,12 @@ pub(super) use c::{preadv64v2 as libc_preadv2, pwritev64v2 as libc_pwritev2};
 #[cfg(any(target_os = "ios", target_os = "macos"))]
 pub(super) use readwrite_pv::{preadv as libc_preadv, pwritev as libc_pwritev};
 
-#[cfg(not(any(
-    windows,
-    target_os = "android",
-    target_os = "linux",
-    target_os = "emscripten",
-    target_os = "l4re",
-    target_os = "netbsd",
-    target_os = "redox",
-    target_os = "wasi",
-)))]
-pub(super) use c::fstatfs as libc_fstatfs;
 #[cfg(not(any(
     windows,
     target_os = "android",
     target_os = "dragonfly",
     target_os = "fuchsia",
+    target_os = "illumos",
     target_os = "ios",
     target_os = "linux",
     target_os = "l4re",
@@ -192,8 +183,20 @@ pub(super) use c::fstatfs as libc_fstatfs;
     target_os = "redox",
 )))]
 pub(super) use c::posix_fallocate as libc_posix_fallocate;
-#[cfg(any(target_os = "l4re",))]
+#[cfg(any(target_os = "l4re"))]
 pub(super) use c::posix_fallocate64 as libc_posix_fallocate;
+#[cfg(not(any(
+    windows,
+    target_os = "android",
+    target_os = "emscripten",
+    target_os = "illumos",
+    target_os = "linux",
+    target_os = "l4re",
+    target_os = "netbsd",
+    target_os = "redox",
+    target_os = "wasi",
+)))]
+pub(super) use {c::fstatfs as libc_fstatfs, c::statfs as libc_statfs};
 
 #[cfg(any(
     target_os = "android",
@@ -201,4 +204,4 @@ pub(super) use c::posix_fallocate64 as libc_posix_fallocate;
     target_os = "emscripten",
     target_os = "l4re",
 ))]
-pub(super) use c::fstatfs64 as libc_fstatfs;
+pub(super) use {c::fstatfs64 as libc_fstatfs, c::statfs64 as libc_statfs};
