@@ -1,3 +1,5 @@
+//! Tests for [`rustix::io`].
+
 #![cfg_attr(target_os = "wasi", feature(wasi_ext))]
 #![cfg_attr(io_lifetimes_use_std, feature(io_safety))]
 
@@ -17,8 +19,10 @@ mod isatty;
 #[cfg(not(windows))]
 mod mmap;
 #[cfg(not(windows))]
+mod poll;
+#[cfg(not(windows))]
 mod prot;
 #[cfg(not(windows))]
 #[cfg(not(target_os = "redox"))] // redox doesn't have cwd/openat
 #[cfg(not(target_os = "wasi"))] // wasi support for S_IRUSR etc. submitted to libc in #2264
-mod readwrite;
+mod read_write;
